/*
 * @(#)file      MLet.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.40
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management.loading;


// Java import
import java.net.URLClassLoader;
import java.net.URL;
import java.net.URLStreamHandlerFactory;
import java.net.MalformedURLException;
import java.util.Set;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import java.util.Arrays;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.ObjectInputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.lang.reflect.Constructor;

// RI import
import javax.management.ServiceNotFoundException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.ReflectionException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanException;
import javax.management.NotCompliantMBeanException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistration;
import com.sun.management.jmx.Trace;
import com.sun.management.jmx.ServiceName;
import com.sun.management.jmx.JMProperties;


/**
 * Allows you to instantiate and register one or several MBeans in the MBean server 
 * coming from a remote URL. M-let is a shortcut for management applet. The m-let service does this 
 * by loading an m-let text file, which specifies information on the MBeans to be obtained. 
 * The information on each MBean is specified in a single instance of a tag, called the MLET tag. 
 * The location of the m-let text file is specified by a URL. 
 * <p>
 * The <CODE>MLET</CODE> tag has the following syntax:
 * <p>
 * &lt;<CODE>MLET</CODE><BR>
 *      <CODE>CODE = </CODE><VAR>class</VAR><CODE> | OBJECT = </CODE><VAR>serfile</VAR><BR>
 *      <CODE>ARCHIVE = &quot;</CODE><VAR>archiveList</VAR><CODE>&quot;</CODE><BR>
 *      <CODE>[CODEBASE = </CODE><VAR>codebaseURL</VAR><CODE>]</CODE><BR>
 *      <CODE>[NAME = </CODE><VAR>mbeanname</VAR><CODE>]</CODE><BR>
 *      <CODE>[VERSION = </CODE><VAR>version</VAR><CODE>]</CODE><BR>
 * &gt;<BR>
 *	<CODE>[</CODE><VAR>arglist</VAR><CODE>]</CODE><BR>
 * &lt;<CODE>/MLET</CODE>&gt;
 * <p>
 * where:
 * <DL>
 * <DT><CODE>CODE = </CODE><VAR>class</VAR></DT>
 * <DD>
 * This attribute specifies the full Java class name, including package name, of the MBean to be obtained.
 * The compiled <CODE>.class</CODE> file of the MBean must be contained in one of the <CODE>.jar</CODE> files specified by the <CODE>ARCHIVE</CODE>
 * attribute. Either <CODE>CODE</CODE> or <CODE>OBJECT</CODE> must be present.
 * </DD>
 * <DT><CODE>OBJECT = </CODE><VAR>serfile</VAR></DT>
 * <DD>
 * This attribute specifies the <CODE>.ser</CODE> file that contains a serialized representation of the MBean to be obtained.
 * This file must be contained in one of the <CODE>.jar</CODE> files specified by the <CODE>ARCHIVE</CODE> attribute. If the <CODE>.jar</CODE> file contains a directory hierarchy, specify the path of the file within this hierarchy. Otherwise  a match will not be found. Either <CODE>CODE</CODE> or <CODE>OBJECT</CODE> must be present.
 * </DD>
 * <DT><CODE>ARCHIVE = &quot;</CODE><VAR>archiveList</VAR><CODE>&quot;</CODE></DT>
 * <DD>
 * This mandatory attribute specifies one or more <CODE>.jar</CODE> files 
 * containing MBeans or other resources used by
 * the MBean to be obtained. One of the <CODE>.jar</CODE> files must contain the file specified by the <CODE>CODE</CODE> or <CODE>OBJECT</CODE> attribute.
 * If archivelist contains more than one file:
 * <UL>
 * <LI>Each file must be separated from the one that follows it by a comma (,).
 * <LI><VAR>archivelist</VAR> must be enclosed in double quote marks.
 * </UL>
 * All <CODE>.jar</CODE> files in <VAR>archivelist</VAR> must be stored in the directory specified by the code base URL.
 * </DD>
 * <DT><CODE>CODEBASE = </CODE><VAR>codebaseURL</VAR></DT>
 * <DD>
 * This optional attribute specifies the code base URL of the MBean to be obtained. It identifies the directory that contains
 * the <CODE>.jar</CODE> files specified by the <CODE>ARCHIVE</CODE> attribute. Specify this attribute only if the <CODE>.jar</CODE> files are not in the same
 * directory as the m-let text file. If this attribute is not specified, the base URL of the m-let text file is used.
 * </DD>
 * <DT><CODE>NAME = </CODE><VAR>mbeanname</VAR></DT>
 * <DD>
 * This optional attribute specifies the object name to be assigned to the
 * MBean instance when the m-let service registers it. If 
 * <VAR>mbeanname</VAR> starts with the colon character (:), the domain 
 * part of the object name is the domain of the agent. The m-let service 
 * invokes the <CODE>getDomain()</CODE> method of the Framework class to 
 * obtain this information.
 * </DD>
 * <DT><CODE>VERSION = </CODE><VAR>version</VAR></DT>
 * <DD>
 * This optional attribute specifies the version number of the MBean and 
 * associated <CODE>.jar</CODE> files to be obtained. This version number can 
 * be used to specify that the <CODE>.jar</CODE> files are loaded from the 
 * server to update those stored locally in the cache the next time the m-let
 * text file is loaded. <VAR>version</VAR> must be a series of non-negative 
 * decimal integers each separated by a period from the one that precedes it.
 * </DD>
 * <DT><VAR>arglist</VAR></DT>
 * <DD>
 * This optional attribute specifies a list of one or more parameters for the 
 * MBean to be instantiated. This list describes the parameters to be passed the MBean's constructor.
 * Use the following syntax to specify each item in
 * <VAR>arglist</VAR>:</DD>
 * <DL>
 * <P>
 * <DT>&lt;<CODE>ARG TYPE=</CODE><VAR>argumentType</VAR> <CODE>VALUE=</CODE><VAR>value</VAR>&gt;</DT>
 * <P>
 * <DD>where:</DD>
 * <UL>
 * <LI><VAR>argumentType</VAR> is the type of the argument that will be passed as parameter to the MBean's constructor.</UL>
 * </DL>
 * <P>The arguments' type in the argument list should be a Java primitive type or a Java basic type
 * (<CODE>java.lang.Boolean, java.lang.Byte, java.lang.Short, java.lang.Long, java.lang.Integer, java.lang.Float, java.lang.Double, java.lang.String</CODE>).
 * </DL>
 *
 * When an m-let text file is loaded, an 
 * instance of each MBean specified in the file is created and registered.
 * <P>
 * The m-let Service extends the <CODE>java.net.URLClassLoader</CODE> and can be used to load remote classes
 * and jar files in the VM of the agent.
 * <p><STRONG>Note - </STRONG> The <CODE>MLet</CODE> class loader uses the {@link javax.management.loading.DefaultLoaderRepository DefaultLoaderRepository}
 * to load classes that could not be found in the loaded jar files.
 */
 public class MLet extends java.net.URLClassLoader implements MLetMBean, MBeanRegistration   { 


     /*
     * ------------------------------------------
     *   PRIVATE VARIABLES
     * ------------------------------------------
     */  
     
     /**
      * The reference to the MBean server.
      * @serial
      */
     private MBeanServer server = null;

     
     /**
     * The list of instances of the <CODE>MLetContant</CODE> class found at the specified URL.
     * @serial
     */
     private Vector mletList = new Vector();
     
   
     /**
      * The directory used for storing libraries locally before they are loaded.
      */
     private String libraryDirectory = System.getProperty("user.dir");
     
     
     /**
      * The object name of the MLet Service.
      * @serial
      */
     private ObjectName mletObjectName = null;
     
     /**
      * The URLs of the MLet Service.
      * @serial
      */
     private URL[] myUrls = null;
     
     /**
      * System parameters
      */
     private static Hashtable systemParam = new Hashtable(7);
    static {
	 systemParam.put("archive", "archive");
	 systemParam.put("code", "code");
	 systemParam.put("codebase", "codebase");
	 systemParam.put("name", "name");
	 systemParam.put("object", "object");
	 systemParam.put("persistent", "persistent");
	 systemParam.put("version", "version");
    }
     /**
      * Externalize tags.
      */
     private final static byte TE_NULL = (byte)0x70; // null object reference
     private final static byte TE_OBJECT = (byte)0x71; // new object
     
     /**
      * The name of this class to be used for trace messages
      */
     private String dbgTag = "MLet";

     /**
      * Tag used to indicate if the the Default Loader Repository
      * asks the MLet to load classes
      */
     private boolean dlr = false;
     
     /**
      * objects maps from primitive classes to primitive object classes.
      */
     private  Hashtable primitiveClasses = new Hashtable(8) ;
     {
	 primitiveClasses.put(Boolean.TYPE.toString(), getClass("java.lang.Boolean"));
	 primitiveClasses.put(Character.TYPE.toString(), getClass("java.lang.Character"));
	 primitiveClasses.put(Byte.TYPE.toString(), getClass("java.lang.Byte"));
	 primitiveClasses.put(Short.TYPE.toString(), getClass("java.lang.Short"));
	 primitiveClasses.put(Integer.TYPE.toString(), getClass("java.lang.Integer"));
	 primitiveClasses.put(Long.TYPE.toString(), getClass("java.lang.Long"));
	 primitiveClasses.put(Float.TYPE.toString(), getClass("java.lang.Float"));
	 primitiveClasses.put(Double.TYPE.toString(), getClass("java.lang.Double"));
	 
     }
 

     /*
      * ------------------------------------------
      *  CONSTRUCTORS
      * ------------------------------------------
      */
     
     /**
      * Constructs a new MLet using the default delegation parent ClassLoader.
      */     
     public MLet() {
	 super(new URL[0]);
	 if ((libraryDirectory = (String) System.getProperty(JMProperties.MLET_LIB_DIR)) == null) {
	     libraryDirectory = System.getProperty("user.dir");
	 }
     }

     /**
      * Constructs a new MLet for the specified URLs using the default delegation parent ClassLoader.
      * The URLs will be searched in the order specified for classes and resources after first searching in the parent
      * class loader. 
      *
      * @param  urls  The URLs from which to load classes and resources.
      *
      */     
     public MLet(URL[] urls) {
	 super(urls);
	 if ((libraryDirectory = (String) System.getProperty(JMProperties.MLET_LIB_DIR)) == null) {
	     libraryDirectory = System.getProperty("user.dir");
	 }
     }
     
     /**
      * Constructs a new MLet  for the given URLs. The URLs will be searched in the order specified for
      * classes and resources after first searching in the specified parent class loader. 
      * The parent argument will be used as the parent class loader for delegation. The
      * factory argument will be used as the stream handler factory to obtain protocol handlers when creating new
      * URLs. 
      *
      * @param  urls  The URLs from which to load classes and resources.
      * @param  parent The parent class loader for delegation.
      *
      */     
     public MLet(URL[] urls, ClassLoader parent) {
	 super(urls, parent);
	 if ((libraryDirectory = (String) System.getProperty(JMProperties.MLET_LIB_DIR)) == null) {
	     libraryDirectory = System.getProperty("user.dir");
	 }
     }
   
     /**
      * Constructs a new MLet for the specified URLs, parent class loader, and
      * URLStreamHandlerFactory. The parent argument will be used as the parent class loader for delegation. The
      * factory argument will be used as the stream handler factory to obtain protocol handlers when creating new
      * URLs. 
      *
      * @param  urls  The URLs from which to load classes and resources.
      * @param  parent The parent class loader for delegation.
      * @param  factory  The URLStreamHandlerFactory to use when creating URLs.
      *
      */     
     public MLet(URL[] urls, ClassLoader parent, URLStreamHandlerFactory factory) {
	 super(urls, parent, factory);
	 if ((libraryDirectory = (String) System.getProperty(JMProperties.MLET_LIB_DIR)) == null) {
	     libraryDirectory = System.getProperty("user.dir");
	 }
     }

     
     /*
      * ------------------------------------------
      *  PUBLIC METHODS
      * ------------------------------------------
      */
     
     
     /**
      * Appends the specified URL to the list of URLs to search for classes and
      * resources.
      */
     public void addURL(URL url) {
	 if (!Arrays.asList(getURLs()).contains(url))
	     super.addURL(url);
     }

     /**
      * Appends the specified URL to the list of URLs to search for classes and
      * resources.
      * @exception ServiceNotFoundException The specified URL is malformed.
      */
     public void addURL(String url) throws ServiceNotFoundException {
	 try {
	     URL ur = new URL(url);
	     if (!Arrays.asList(getURLs()).contains(ur))
		 super.addURL(ur);
	 } catch (MalformedURLException e) {
	     throw new ServiceNotFoundException("The specified URL is malformed");
	 }
     }
      
     /** Returns the search path of URLs for loading classes and resources. 
      * This includes the original list of URLs specified to the constructor, 
      * along with any URLs subsequently appended by the addURL() method.
      */
     public URL[] getURLs() {
	 return super.getURLs();
     }

     /**
      * Loads a text file containing MLET tags that define the MBeans to
      * be added to the agent. The location of the text file is specified by
      * a URL. The MBeans specified in the MLET file will be instantiated and
      * registered by the MBean server.
      *
      * @param url The URL of the text file to be loaded as URL object.
      *
      * @return  A set containing one entry per MLET tag in the m-let text file loaded.
      * Each entry specifies either the ObjectInstance for the created MBean, or a throwable object
      * (that is, an error or an exception) if the MBean could not be created.
      *       
      * @exception ServiceNotFoundException One of the following errors has occurred: The m-let text file does
      * not contain an MLET tag, the m-let text file is not found, a mandatory
      * attribute of the MLET tag is not specified, the value of url is
      * null.
      */      
     public Set getMBeansFromURL(URL url) throws ServiceNotFoundException  { 
	 if (url == null) {
	     throw new ServiceNotFoundException("The specified URL is null");
	 }
	 return getMBeansFromURL(url.toString());
     } 

     /**
      * Loads a text file containing MLET tags that define the MBeans to
      * be added to the agent. The location of the text file is specified by
      * a URL. The MBeans specified in the MLET file will be instantiated and
      * registered by the MBean server.
      *
      * @param url The URL of the text file to be loaded as String object.
      *
      * @return  A set containing one entry per MLET tag in the m-let text file loaded.
      * Each entry specifies either the ObjectInstance for the created MBean, or a throwable object
      * (that is, an error or an exception) if the MBean could not be created.
      *       
      * @exception ServiceNotFoundException One of the following errors has occurred: The m-let text file does
      * not contain an MLET tag, the m-let text file is not found, a mandatory
      * attribute of the MLET tag is not specified, the url is malformed.
      *
      */     
     public Set getMBeansFromURL(String url) throws ServiceNotFoundException  { 
	 String mth = "getMBeansFromURL";
	 // Parse arguments     
	 if (url == null) {
	     if (isTraceOn()) {
		 trace(mth, "URL is null");
	     }  
	     throw new ServiceNotFoundException("The specified URL is null");
	 } else {
	     url = url.replace(File.separatorChar,'/');
	 }
	 if (isTraceOn()) {
	     trace(mth, "<URL = " + url + ">");
	 }       
	 // Parse URL    
	 MLetParser parser = new MLetParser();
	 mletList = parser.parseURL(url);
	 if (mletList == null) {
	     if (isTraceOn()) {
		 trace(mth, "Problems while parsing URL " + url);
	     }  
	     throw new ServiceNotFoundException("Problems while parsing URL " + url);
	 }
	 
	 // Check that the list of MLets is not empty     
	 if (mletList.size() == 0) {
	     if (isTraceOn()) {
		 trace(mth, "File " + url + " not found or MLET tag not defined in file");
	     }       
	     throw new ServiceNotFoundException("File " + url + " not found or MLET tag not defined in file");
	 }
	 
	 // Walk through the list of MLets        
	 HashSet mbeans = new HashSet();
	 for(Enumeration e = mletList.elements(); e.hasMoreElements(); ) {	 
	     // Get MLet item from list	 
	     MLetContent elmt = (MLetContent) e.nextElement();	 
	     // Initialise local variables            
	     String code = elmt.getCode();
	     if (code != null) {
		 if (code.endsWith(".class")) {
		     code = code.substring(0, code.length() - 6);
		 }
	     }
	     String name = elmt.getName();
	     URL codebase = elmt.getCodeBase();
	     String version = elmt.getVersion();    
	     String serName = elmt.getSerializedObject(); 
	     String jarFiles = elmt.getJarFiles(); 
	     URL documentBase = elmt.getDocumentBase();
	     Hashtable attributes = elmt.getAttributes();
	     
	     // Display debug information
	     if (isTraceOn()) {
		 trace(mth, "MLET TAG     = " + attributes.toString());
		 trace(mth, "CODEBASE     = " + codebase);
		 trace(mth, "ARCHIVE      = " + jarFiles);
		 trace(mth, "CODE         = " + code);
		 trace(mth, "OBJECT       = " + serName);
		 trace(mth, "NAME         = " + name);
		 trace(mth, "VERSION      = " + version);
		 trace(mth, "DOCUMENT URL = " + documentBase);
	     }
	     
	     // Load classes from JAR files	    
	     StringTokenizer st = new StringTokenizer(jarFiles, ",", false);
	     while (st.hasMoreTokens()) {
		 String tok = st.nextToken().trim();
		 if (isTraceOn()) {
		     trace(mth, "Load archive for codebase <" + codebase + ">, file <" + tok + ">");
		 }
		 // Check which is the codebase to be used for loading the jar file.
		 // If we are using the base MLet implementation then it will be
		 // always the remote server but if the service has been extended in
		 // order to support caching and versioning then this method will
		 // return the appropriate one.
		 //
		 try {
		     codebase = check(version, codebase, tok, elmt);
		 } catch (Exception ce) {
		     if (isDebugOn()) {
			 debug(mth, "Versioning Error: Versioning won't be performed");
		     }  
		     codebase = elmt.getCodeBase();
		 }
		 
		 // Appends the specified JAR file URL to the list of URLs to search for classes and resources.
		 try {
		     if (!Arrays.asList(getURLs()).contains(new URL(codebase.toString() + tok))) {
			 addURL(codebase + tok);
		     }	     
		 } catch (MalformedURLException me) {}
		 
	     }
	     // Instantiate the class specified in the
	     // CODE or OBJECT section of the MLet tag
	     //
	     Object o = null;
	     ObjectInstance objInst = null;
	     
	     if (code != null && serName != null) {
		 if (isTraceOn()) {
		     trace(mth, "CODE and OBJECT parameters cannot be specified at the same time in tag MLET.");
		 }
		 mbeans.add(new Error("CODE and OBJECT parameters cannot be specified at the same time in tag MLET"));
		 continue;
	     }
	     if (code == null && serName == null) {
		 if (isTraceOn()) {
		     trace(mth, "Either CODE or OBJECT parameter must be specified in tag MLET.");
		 }
		 mbeans.add(new Error("Either CODE or OBJECT parameter must be specified in tag MLET"));
		 continue;
	     }
	     try {
		 if (code != null) {
		     Hashtable atts = new Hashtable();
		     Vector signat = new Vector();
		     Vector pars = new Vector();
		     
		     for (Enumeration p = attributes.keys(); p.hasMoreElements(); ) {	
			 String param_name = (String) p.nextElement();
			 if (param_name.equals("types")) {
			     signat = (Vector)elmt.getParameter(param_name);
			 }
			 if (param_name.equals("values")) {
			     pars = (Vector)elmt.getParameter(param_name);
			 }			 
		     }
		     for (int i = 0; i < signat.size(); i++) {
			 pars.setElementAt(constructParameter((String)pars.elementAt(i), (String)signat.elementAt(i)), i);
		     }	
		     if (signat.isEmpty()) {
			 if (name == null) {
			     objInst = server.createMBean(code, null, mletObjectName); 
			 } else {
			     objInst = server.createMBean(code, new ObjectName(name), mletObjectName); 
			 }
		     } else {		     
			 Object[] parms = pars.toArray();
			 String[] signature = new String[signat.size()];
			 for (int i=0;i<signature.length;i++) {
			     signature[i] = (String) signat.elementAt(i);
			 }
			 if (isDebugOn()) {
			     for (int i=0;i<signature.length;i++) {
				 debug(mth, "Signature     = " + signature[i]);
				 debug(mth, "Params     = " + parms[i].toString());
			     }
			 }	
			 if (name == null) {
			     objInst = server.createMBean(code, null, mletObjectName, parms, signature); 
			 } else {
			     objInst = server.createMBean(code, new ObjectName(name), mletObjectName, parms, signature); 
			 }
		     } 		    
		 } else {
		     o = loadSerializedObject(codebase,serName);
		     if (name == null) {
			 server.registerMBean(o, null);
		     } else {
			 server.registerMBean(o,  new ObjectName(name));
		     }
		     objInst = new ObjectInstance(name, o.getClass().getName());
		 }
	     } catch (ReflectionException  ex) {
		 if (isTraceOn()) {
		     trace(mth, "ReflectionException: " + ex.getMessage());
		 }	
		 mbeans.add(ex);
		 continue;
	     } catch (InstanceAlreadyExistsException  ex) {
		 if (isTraceOn()) {
		     trace(mth, "InstanceAlreadyExistsException: " + ex.getMessage());
		 }	
		 mbeans.add(ex);
		 continue;
	     } catch (MBeanRegistrationException ex) {      
		 if (isTraceOn()) {
		     trace(mth, "MBeanRegistrationException: " + ex.getMessage());
		 }
		 mbeans.add(ex);
		 continue;
	     } catch (MBeanException  ex) {
		 if (isTraceOn()) {
		     trace(mth, "MBeanException: " + ex.getMessage());
		 }      
		 mbeans.add(ex);
		 continue;
	     } catch (NotCompliantMBeanException  ex) {      
		 if (isTraceOn()) {
		     trace(mth, "NotCompliantMBeanException: " + ex.getMessage());
		 }  
		 mbeans.add(ex);
		 continue;
	     } catch (InstanceNotFoundException   ex) { 
		 if (isTraceOn()) {
		     trace(mth, "InstanceNotFoundException: " + ex.getMessage());
		 }  
		 mbeans.add(ex);
		 continue;
	     } catch (IOException ex) {
		 if (isTraceOn()) {
		     trace(mth, "IOException: " + ex.getMessage());
		 }  
		 mbeans.add(ex);
		 continue;
	     } catch (SecurityException ex) {
		 if (isTraceOn()) {
		     trace(mth, "SecurityException: " + ex.getMessage());
		 } 
		 mbeans.add(ex);
		 continue;
	     } catch (Exception ex) {
		 if (isTraceOn()) {
		     trace(mth, "Exception: " + ex.getClass().getName() + ex.getMessage());
		 } 
		 mbeans.add(ex);
		 continue;
	     } catch (Error ex) {
		 if (isTraceOn()) {
		     trace(mth, "Error: " + ex.getMessage());
		 } 
		 mbeans.add(ex);
		 continue;
	     }
	     mbeans.add(objInst);
	 }
	 return mbeans;
     }
     
     /**
      * Gets the current directory used by the library loader for
      * storing native libraries before they are loaded into memory.
      *
      * @return The current directory used by the library loader.
      */
     public String getLibraryDirectory() {
	 return libraryDirectory;
     }
     
     /**
      * Sets the directory used by the library loader for storing
      * native libraries before they are loaded into memory.
      * @param <VAR>libdir</VAR> The directory used by the library loader.
      */
     public void setLibraryDirectory(String libdir) {
	 libraryDirectory = libdir;
     }
          
     /**
      * Allows the m-let to perform any operations it needs before being registered
      * in the MBean server. If the ObjectName is null, the m-let provides a 
      * default name for its registration <defaultDomain>:type=MLet
      *
      * @param server The MBean server in which the m-let will be registered.
      * @param name The object name of the m-let.
      *
      * @return  The name of the m-let registered.
      *
      * @exception java.langException This exception should be caught by the MBean server and re-thrown
      *as an MBeanRegistrationException.
      */    
     public ObjectName preRegister(MBeanServer server, ObjectName name) throws java.lang.Exception {
	 
	 // Initialise local pointer to the MBean server
	 this.server = server;
	 this.mletObjectName = name;
	 
	 // If no name is specified return a default name for the MLet
	 if (name == null) {	     
	     return new ObjectName(server.getDefaultDomain() + ":" + ServiceName.MLET);	     
	 } else {
	     return name;
	 }
     }
     
     /**
      * Allows the m-let to perform any operations needed after having been
      * registered in the MBean server or after the registration has failed.
      *
      * @param registrationDone Indicates whether or not the m-let has been successfully registered in
      * the MBean server. The value false means that either the registration phase
      * has failed.
      *
      */
     public void postRegister (Boolean registrationDone) {
     }
          
     /**
      * Allows the m-let to perform any operations it needs before being de-registered
      * by the MBean server.
      *
      * @exception java.langException  This exception should be caught by the MBean server and re-thrown
      * as an MBeanRegistrationException.
      */     
     public void preDeregister() throws java.lang.Exception {
     }
     
     
     /**
      * Allows the m-let to perform any operations needed after having been
      * de-registered in the MBean server.
      */
     public void postDeregister() {
     }
     
     
     // Externalizable
     //----------------
     
     /**
      * This method saves the object's contents during Object Serialization.
      *
      * @param <VAR>out</VAR> The serialization output stream.
      *
      * @exception IOException Signals that an I/O exception of some sort has occurred.
      */
     public void writeExternal(ObjectOutput out)
	 throws IOException {
	 
	 // Write object contents to stream.
	 //
	 writeContents(out, super.getURLs()); // write code base
	 writeContents(out, mletObjectName); // write object name
	 writeContents(out, server); // write core management framework
	 writeContents(out, libraryDirectory); // write library directory
     }
     
     /**
      * This method reads the object's contents during Object Serialization.
      *
      * @param <VAR>in</VAR> The serialization input stream.
      *
      * @exception IOException Signals that an I/O exception of some sort has occurred.
      * @exception ClassNotFoundException The class for an object being restored cannot be found.
      */
     public void readExternal(ObjectInput in)
	 throws IOException, ClassNotFoundException {
	 
	 // Read object contents from stream.
        //
	 URL[] myUrls = (URL[]) readContents(in); // read code base
	 if (myUrls != null) {
	     for (int i = 0; i < myUrls.length; i++) {
		 addURL(myUrls[i]);
	     }
	 }
	 mletObjectName = (ObjectName) readContents(in); // read object name
	 server = (MBeanServer) readContents(in); // read core management framework
	 libraryDirectory = (String) readContents(in); // read library directory	 
    }

     /*
      * ------------------------------------------
      *  PACKAGE METHODS
      * ------------------------------------------
      */
     
     Class loadClassCalledFromDLR(String name) throws ClassNotFoundException {
	 dlr = true;
	 return(loadClass(name));
     }
     /*
      * ------------------------------------------
      *  PROTECTED METHODS
      * ------------------------------------------
      */
     
     /**
      * This is the main method for class loaders that is being redefined.
      *
      * @param <VAR>name</VAR> The name of the class.
      *
      * @return The resulting Class object.
      *
      * @exception ClassNotFoundException The specified class could not be found.
      */
     protected Class findClass(String name) throws ClassNotFoundException {
	 
	 Class c = null;
	 if (isTraceOn()) {
	     trace("findClass", name);
	 }   
	 // Try looking in the JAR:
	 try {
	     c = super.findClass(name);
	     if (isTraceOn()) {
		 trace("findClass", "Class "+name+" loaded through mlet classloader");
	     }     
	 } catch (ClassNotFoundException e) {
	     // Drop through
	 }
	 
	 // if we are not called from the Default Loader Repository
	 if (!dlr) {
	     // Try the classloader repository:	     
	     if (c == null) {
		 try {
		     c = DefaultLoaderRepository.loadClassWithout(this, name);
		     // The loadClassWithout method never returns null.
		     // If the class is not found we get an exception.
		     if (isTraceOn()) {
			 trace("findClass", "Class "+name+" loaded through the default classloader repository");
		     }
		 } catch (ClassNotFoundException e) {
		     // Drop through
		 }
	     }
	 }
	 if (c == null) {
	     throw new ClassNotFoundException(name);
	 }
	 return c;
     }

     /**
      * Returns the absolute path name of a native library. The VM invokes this method to locate the native
      * libraries that belong to classes loaded with this class loader. Libraries are searched in the JAR files
      * using first just the native library name and if not found the native library name together with the
      * architecture-specific path name (<code>OSName/OSArch/OSVersion/lib/nativelibname</code>), i.e.
      * <p>
      * the library stat on Solaris SPARC 5.7 will be searched in the JAR file as:
      * <OL>
      * <LI>libstat.so
      * <LI>SunOS/sparc/5.7/lib/libstat.so
      * </OL>
      * the library stat on Windows NT 4.0 will be searched in the JAR file as:
      * <OL>
      * <LI>stat.dll
      * <LI>WindowsNT/x86/4.0/lib/stat.dll
      * </OL>
      * <p>
      * If this method returns <code>null</code>, i.e. the libraries were not found in any of the JAR
      * files loaded with this class loader, the VM searches the library along the path specified as
      * the <code>java.library.path</code> property.
      *
      * @param libname The library name.
      *
      * @return The absolute path of the native library.
      */
     protected String findLibrary(String libname) {
	 
	 String abs_path;
	 String mth = "findLibrary";
	 
	 // Get the platform-specific string representing a native library.
	 //
	 String nativelibname = System.mapLibraryName(libname);
	 
	 //
	 // See if the native library is accessible as a resource through the JAR file.
	 //
	 if (isTraceOn()) {
	     trace(mth, "Search " + libname + " in all JAR files.");
	 }       
	 
	 // First try to locate the library in the JAR file using only the native library name.
	 // e.g. if user requested a load for "foo" on Solaris SPARC 5.7 we try to load
	 //      "libfoo.so" from the JAR file.
	 //
	 if (isTraceOn()) {
	     trace(mth, "loadLibraryAsResource(" + nativelibname + ")");
	 }        
	 abs_path = loadLibraryAsResource(nativelibname);
	 if (abs_path != null) {
	     if (isTraceOn()) {
		 trace(mth, nativelibname + " loaded " + "absolute path = " + abs_path);
	     }             
	     return abs_path;
	 }

	 // Next try to locate it using the native library name and the architecture-specific path name.
	 // e.g. if user requested a load for "foo" on Solaris SPARC 5.7 we try to load
	 //      "SunOS/sparc/5.7/lib/libfoo.so" from the JAR file.
	 //
	 nativelibname = removeSpace(System.getProperty("os.name")) + File.separator +
	     removeSpace(System.getProperty("os.arch")) + File.separator +
	     removeSpace(System.getProperty("os.version")) + File.separator +
	     "lib" + File.separator + nativelibname;
	 if (isTraceOn()) {
	     trace(mth, "loadLibraryAsResource(" + nativelibname + ")");
	 }     
	 
	 abs_path = loadLibraryAsResource(nativelibname);
	 if (abs_path != null) {
	     if (isTraceOn()) {
		 trace(mth, nativelibname + " loaded " + "absolute path = " + abs_path);
	     }                
	     return abs_path;
	 }
	 
	 //
	 // All paths exhausted, library not found in JAR file.
	 //
	 
	 if (isTraceOn()) {
	     trace(mth, libname + " not found in any JAR file.");
	     trace(mth, "Search " + libname + " along the path specified as the java.library.path property.");
	 }     
	 
	 
	 // Let the VM search the library along the path
	 // specified as the java.library.path property.
	 //
	 return null;
     }


     /*
      * ------------------------------------------
      *  PRIVATE METHODS
      * ------------------------------------------
      */
     
     /**
      * Search the specified native library in any of the JAR files loaded by this classloader.
      * If the library is found copy it into the library directory and return the absolute path.
      * If the library is not found then return null.
      */
     private synchronized String loadLibraryAsResource(String libname) {
	 try {
	     File file = new File(libraryDirectory, libname);
	     if (file.exists()) {
	     	 file.delete();
	     }	      
	     InputStream is = getResourceAsStream(libname.replace(File.separatorChar,'/'));
	     if (is != null) {
		 file.getParentFile().mkdirs();
		 FileOutputStream fileOutput = new FileOutputStream(file);
		 int c;
		 while ((c = is.read()) != -1) {
		     fileOutput.write(c);
		 }
		 is.close();
		 fileOutput.close();
		 if (file.exists()) {
		     return file.getAbsolutePath();
		 }
	     }
	 } catch (Exception e) {
	     return null;
	 }
	 return null;
     }
     
   /**
    * Removes any white space from a string. This is used to
    * convert strings such as "Windows NT" to "WindowsNT".
    */
     private String removeSpace(String s) {
	 s = s.trim();
	 int j = s.indexOf(' ');
	 if (j == -1) {
	     return s;
	 }
	 String temp = new String();
	 int k = 0;
	 while (j != -1) {
	     s = s.substring(k);
	     j = s.indexOf(' ');
	     if (j != -1) {
		 temp = temp + s.substring(0, j);
	     } else {
		 temp = temp + s.substring(0);
	     }
	     k = j + 1;
	 }
	 return temp;
     }
     
     /**
      * Writes an object to the stream.
      */
     private void writeContents(ObjectOutput out, Object object)
	 throws IOException {
	 
	 if (object == null) {
	     // Null object reference.
	     //
	     out.writeByte(TE_NULL);
	 } else {
	     // Write object.
	     //
	     out.writeByte(TE_OBJECT);
	     out.writeObject(object);
	 }
     }
     
     /**
      * Reads an object from the stream.
      */
     private Object readContents(ObjectInput in)
	 throws IOException, ClassNotFoundException {
	 
	 // Read tag.
	 //
	 byte tag = in.readByte();
	 if (tag == TE_OBJECT) {
	     // New object.
	     //
	     return in.readObject();
	 } else if (tag == TE_NULL) {
	     // Null object reference.
	     //
	     return null;
	 } else {
	     // The stream is corrupted.
	     //
	     throw new IOException("Input stream corrupted.");
	 }
    }     
     
     /**
      * This method is to be overridden when extending this service to support caching and versioning.
      *
      * @param <VAR>version</VAR> The version number of the <CODE>.jar</CODE> file stored locally.
      * @param <VAR>codebase</VAR> The base URL of the remote <CODE>.jar</CODE> file.
      * @param <VAR>jarfile</VAR> The name of the <CODE>.jar</CODE> file to be loaded.
      * @param <VAR>mlet</VAR> The <CODE>MLetContent</CODE> instance that represents the <CODE>MLET</CODE> tag.
      *
      */
     private synchronized URL check(String version, URL codebase, String jarfile, MLetContent mlet) {
	 return codebase;
     }
     	 
    /**
     * Loads the serialized object specified by the <CODE>OBJECT</CODE> 
     * attribute of the <CODE>MLET</CODE> tag.
     *
     * @param <VAR>codebase</VAR> The <CODE>codebase</CODE>.
     * @param <VAR>filename</VAR> The name of the file containing the serialized object.
     * @return The serialized object.
     * @exception ClassNotFoundException The specified serialized object could  not be found.
     * @exception IOException An I/O error occurred while loading serialized object.
     */
     private Object loadSerializedObject(URL codebase, String filename) throws IOException, ClassNotFoundException {
        if (filename != null) {
            filename = filename.replace(File.separatorChar,'/');
        }		
	if (isTraceOn()) {
	    trace("loadSerializedObject", codebase.toString() + filename);
	}	
        InputStream is = getResourceAsStream(filename);
        if (is != null) {
            try {
                ObjectInputStream ois = new MLetObjectInputStream(is, this);
                Object serObject = ois.readObject();
                ois.close();
                return serObject;
            } catch (IOException e) {
		if (isDebugOn()) {
		    debug("loadSerializedObject", "Exception while deserializing " + filename + ", " + e.getMessage());
		}
                throw e;
            } catch (ClassNotFoundException e) {
		if (isDebugOn()) {
		    debug("loadSerializedObject", "Exception while deserializing " + filename + ", " + e.getMessage());
		}
                throw e;
            }
        } else {
	    if (isDebugOn()) {
		debug("loadSerializedObject", "Error: File " + filename + " containing serialized object not found");
	    }
            throw new Error("File " + filename + " containing serialized object not found");
        }
     }

     /**
      * Converts the String value of the constructor's parameter to 
      * a basic Java object with the type of the parameter.
      */
     private  Object constructParameter(String param, String type) {
	 // check if it is a primitive type
	 Class c = (Class) primitiveClasses.get(type);
	 if (c != null) {
	    Class[] cs = new Class[1];	
	    try {
		cs[0] = Class.forName("java.lang.String");
		Constructor cons = c.getConstructor(cs);	
		Object[] oo = new Object[1];
		oo[0]=param;
		return(cons.newInstance(oo));

	    } catch (Exception  e) {
		if (isDebugOn()) {
		    debug(dbgTag, "constructParameter", "Unexpected Exception" + e.getClass().getName() + " occured");
		}
	    }
	}
	if (type.compareTo("java.lang.Boolean") == 0) 
	     return new Boolean(param);
	if (type.compareTo("java.lang.Byte") == 0) 
	     return new Byte(param);
	if (type.compareTo("java.lang.Short") == 0) 
	     return new Short(param);
	if (type.compareTo("java.lang.Long") == 0) 
	     return new Long(param);
	if (type.compareTo("java.lang.Integer") == 0) 
	     return new Integer(param);
	if (type.compareTo("java.lang.Float") == 0) 
	     return new Float(param);
	if (type.compareTo("java.lang.Double") == 0) 
	     return new Double(param);
	if (type.compareTo("java.lang.String") == 0) 
	     return param;
	
	return param;
     }


     /**
      * Gets the Class object associated with the class with the given
      * string name.
      */
     private Class getClass(String s) {
	 try {
	     return Class.forName(s);
	 } catch(ClassNotFoundException e) {   
	     if (isDebugOn()) {
		 debug(dbgTag, "getClass", "Unexpected ClassNotFoundException occured");
	     }
	 }
	 return null;
     }
         

     // TRACES & DEBUG
     //---------------
     
     private boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_MLET);
     }
     
     private void trace(String clz, String func, String info) {
	 Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MLET, clz, func, info);
     }
     
     private void trace(String func, String info) {
	 trace(dbgTag, func, info);
     }
     
     private boolean isDebugOn() {
	 return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_MLET);
     }
     
     private void debug(String clz, String func, String info) {
	 Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_MLET, clz, func, info);
     }
     
     private void debug(String func, String info) {
	 debug(dbgTag, func, info);
     }     
 }
