/*
 * @(#)file      MLetMBean.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   4.7
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management.loading;

import java.net.URL;
import java.io.InputStream;
import java.io.IOException;
import java.util.Set;
import java.util.Enumeration;

import javax.management.*;



/**
 * Exposes the remote management interface of the MLet
 * MBean.
 */
public interface MLetMBean   { 

    
    /**
     * Loads a text file containing MLET tags that define the MBeans to
     * be added to the agent. The location of the text file is specified by
     * a URL. The MBeans specified in the MLET file will be instantiated and
     * registered by the MBeanServer.
     *
     * @param url The URL of the text file to be loaded as String object.
     *
     * @return  A set containing one entry per MLET tag in the m-let text file loaded.
     * Each entry specifies either the ObjectInstance for the created MBean, or a throwable object
     * (that is, an error or an exception) if the MBean could not be created.
     *
     * @exception ServiceNotFoundException One of the following errors has occurred: The m-let text file does
     * not contain an MLET tag, the m-let text file is not found, a mandatory
     * attribute of the MLET tag is not specified, the value of url is malformed.
     */
    public Set getMBeansFromURL(String url) throws ServiceNotFoundException ; 

    /**
     * Loads a text file containing MLET tags that define the MBeans to
     * be added to the agent. The location of the text file is specified by
     * a URL. The MBeans specified in the MLET file will be instantiated and
     * registered by the MBeanServer.
     *
     * @param url The URL of the text file to be loaded as URL object.
     *
     * @return  A set containing one entry per MLET tag in the m-let text file loaded.
     * Each entry specifies either the ObjectInstance for the created MBean, or a throwable object
     * (that is, an error or an exception) if the MBean could not be created.     
     *
     * @exception ServiceNotFoundException One of the following errors has occurred: The m-let text file does
     * not contain an MLET tag, the m-let text file is not found, a mandatory
     * attribute of the MLET tag is not specified, the value of url is
     * null.
     */
    public Set getMBeansFromURL(URL url) throws ServiceNotFoundException ; 

    /**
     * Appends the specified URL to the list of URLs to search for classes and
     * resources.
     */
    public void addURL(URL url) ; 

    /**
     * Appends the specified URL to the list of URLs to search for classes and
     * resources.
     * @exception ServiceNotFoundException The specified URL is malformed.
     */
    public void addURL(String url) throws ServiceNotFoundException;

    /** Returns the search path of URLs for loading classes and resources. 
     * This includes the original list of URLs specified to the constructor, 
     * along with any URLs subsequently appended by the addURL() method.
     */
    public URL[] getURLs();
    
    /** Finds the resource with the given name. 
     * A resource is some data (images, audio, text, etc) that can be accessed by class code in a way that is
     *   independent of the location of the code.
     *   The name of a resource is a "/"-separated path name that identifies the resource.
     *
     * @param name The resource name
     *
     * @return  An URL for reading the resource, or null if the resource could not be found or the caller doesn't have adequate privileges to get the
     * resource.
     */
    public URL getResource(String name);

    /** Returns an input stream for reading the specified resource. The search order is described in the documentation for
     *  getResource(String).
     *
     * @param name  The resource name
     * 
     * @return An input stream for reading the resource, or null if the resource could not be foun
     *
     */
    public InputStream getResourceAsStream(String name);

    /** Finds all the resources with the given name. A resource is some data (images, audio, text, etc) that can be accessed by class code in a way
     *  that is independent of the location of the code.
     *  The name of a resource is a "/"-separated path name that identifies the resource.
     *
     * @param name The  resource name.
     * @return  An enumeration of URL to the resource. If no resources could be found, the enumeration will be empty. Resources that the doesn't have
     *  access to will not be in the enumeration.
     */
    public  Enumeration getResources(String name) throws IOException;

    /**
     * Gets the current directory used by the library loader for
     * storing native libraries before they are loaded into memory.
     * @return The current directory used by the library loader.
     */
    public String getLibraryDirectory();
    
    /**
     * Sets the directory used by the library loader for storing
     * native libraries before they are loaded into memory.
     * @param <VAR>libdir</VAR> The directory used by the library loader.
     */
    public void setLibraryDirectory(String libdir);

 }
