/*
 * @(#)file      DefaultLoaderRepository.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package com.sun.management.jmx;


// Java import
import java.io.Serializable;
import java.util.Vector;
import java.util.Hashtable;


/**
 * This class keeps the list of Class Loaders registered in the MBean Server.
 * It provides the necessary methods to load classes using the registered Class Loaders.
 *
 */
class DefaultLoaderRepository extends javax.management.loading.DefaultLoaderRepository implements Serializable {
 
    private static final String dbgTag = "DefaultLoaderRepository";
    
    static void addClassLoader(ClassLoader loader) {
	if (isTraceOn()) {
	    trace("addClassLoader", loader.toString());
	} 
	loaders.addElement(loader);
    }
    
    static void removeClassLoader(ClassLoader loader) {
	if (isTraceOn()) {
	    trace("removeClassLoader", " " + loader.toString());
	} 
	loaders.removeElement(loader);
    }
    
  
    // TRACES & DEBUG
    //---------------

    private static boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_MBEANSERVER);
    }

    private static void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MBEANSERVER, clz, func, info);
    }

    private static void trace(String func, String info) {
        trace(dbgTag, func, info);
    }

    private static boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_MBEANSERVER);
    }

    private static void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_MBEANSERVER, clz, func, info);
    }

    private static void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
 

 }
