/*
 * @(#)file      MetaData.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.37
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package com.sun.management.jmx;



// java import

import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.util.Hashtable;

// RI import
import javax.management.* ; 
 

/**
 * The MetaData class provides local access to the metadata service in
 * an agent.
 */
class MetaData {


    /**
     * The initial class to be analyzed.
     */
    private Class baseClass = null;
    
    /**
     * The bean class to be analyzed.
     */
    private Class beanClass;
    
    /**
     * A MBeanInfo object describing the target beans parent classes.
     */
    private MBeanInfo superBeanInfo;
    
    /**
     * properties maps from String names to MBeanAttribute.
     */
    private java.util.Hashtable attributes = new java.util.Hashtable();
    
    /**
     * operations maps from Method objects to MBeanOperation.
     */
    private java.util.Hashtable operations = new java.util.Hashtable();
    
    /**
     * Cache of public declared methods.
     */
    private static java.util.Hashtable declaredMethodCache = new java.util.Hashtable();
    
    /**
     * Cache of public  methods.
     */
    private static java.util.Hashtable MethodCache = new java.util.Hashtable();
    
    
    private  Hashtable primitiveClasses = new Hashtable(8);
    {
        primitiveClasses.put(Boolean.TYPE.toString(), Boolean.TYPE);
        primitiveClasses.put(Character.TYPE.toString(), Character.TYPE);
        primitiveClasses.put(Byte.TYPE.toString(), Byte.TYPE);
        primitiveClasses.put(Short.TYPE.toString(), Short.TYPE);
        primitiveClasses.put(Integer.TYPE.toString(), Integer.TYPE);
        primitiveClasses.put(Long.TYPE.toString(), Long.TYPE);
        primitiveClasses.put(Float.TYPE.toString(), Float.TYPE);
        primitiveClasses.put(Double.TYPE.toString(), Double.TYPE);
     }

    /**
     * objects maps from primitive classes to primitive object classes.
     */
    private static Hashtable primitiveobjects = new Hashtable();
    {
	primitiveobjects.put(Boolean.TYPE, getClass("java.lang.Boolean"));
	primitiveobjects.put(Character.TYPE, getClass("java.lang.Character"));
	primitiveobjects.put(Byte.TYPE, getClass("java.lang.Byte"));
	primitiveobjects.put(Short.TYPE, getClass("java.lang.Short"));
	primitiveobjects.put(Integer.TYPE, getClass("java.lang.Integer"));
	primitiveobjects.put(Long.TYPE, getClass("java.lang.Long"));
	primitiveobjects.put(Float.TYPE, getClass("java.lang.Float"));
	primitiveobjects.put(Double.TYPE, getClass("java.lang.Double"));
    }
    
    /**
     * Cache of MBeanInfo objects.
     */
    private static  java.util.Hashtable mbeanInfoCache = new java.util.Hashtable();
    
    /**
     * Cache of MBean Interface objects.
     */
    private static  java.util.Hashtable mbeanInterfaceCache = new java.util.Hashtable();



    /**
     * Creates a Metadata Service.
     */
    public MetaData()  {      
    } 
    
    /**
     * Constructs a MetaData.
     */
    private MetaData(Class beanClass, Class baseClass) {	
	this.beanClass = beanClass;
	this.baseClass = baseClass;
	
    }

    
    /** 
     * This methods tests if the MBean is JMX compliant
     */    
    public synchronized void testCompliance(Class c) throws NotCompliantMBeanException {
	MBeanInfo bi = Introspector.testCompliance(c);
	if (bi != null) {
	    if (mbeanInfoCache.get(c) == null) {
		mbeanInfoCache.put(c, bi);
	    }
	}
	Class inter = Introspector.getMBeanInterface(c);
	if (inter != null) {
	    if (mbeanInterfaceCache.get(c) == null) {
		mbeanInterfaceCache.put(c, inter);
	    }
	}
    }

    /** 
     * This methods returns the MBean interface of an MBean
     */    
    public Class getMBeanInterface(Class c) {
	Class inter = (Class)mbeanInterfaceCache.get(c);
	return inter;
    }

    /**
     * This method discovers the attributes and operations that an MBean exposes
     * for management.
     *
     * @param beanClass The class to be analyzed.
     *
     * @return  An instance of MBeanInfo allowing to retrieve all methods and operations
     * of this class.
     *
     * @exception IntrospectionException if an exception occurs during introspection.
     *
     */
    public MBeanInfo getMBeanInfo(Class beanClass) throws IntrospectionException {   	
	// Check the mbean information cache.             
	MBeanInfo bi = (MBeanInfo)mbeanInfoCache.get(beanClass);            
	// Make an independent copy of the MBeanInfo.        
	return (MBeanInfo) bi.clone() ;	  	            
   } 
       
    
    /**
     * Finds a specific method of an object.
     * Returns the method or null if not found
     */    
    Method findMethod(Class classObj, String name, Class parameterTypes[]) {
	Method method=null;   
	try {
	    method= classObj.getMethod(name, parameterTypes); 
	} catch(Exception e) {
	}
	
	return method;
    }
    
    /**
     * Finds a specific method of an object without knowing the parameter types.
     * Returns the method or null if not found
     */
    Method findMethod(Class classObj, String name) {
	Method method = null ;
	
	try {
	    Method[] methods=classObj.getMethods();
	    int i = 0;
	    while ((i < methods.length) && !methods[i].getName().equals(name)) {
		i++;
	    }
	    if (i < methods.length) { 
		method = methods[i];
	    }
	} catch(Exception e) {
	}
	return method;
    }

    /**
     * Finds the getter of a specific attribute in an object.
     * Returns the method for accessing the attributes, null otherwise
     */
    Method findGetter(Class classObj, String attribute)  {
	// A getter should not have any parameter	
	Method m = findMethod(classObj, "get" + attribute, null);
	if (m == null) {
	    m = findMethod(classObj, "is" + attribute, null);
	}
     return m;
    }
    
    /**
     * Finds the setter of a specific attribute in an object.
     * Returns the method for accessing the attribute, null otherwise
     */
    
    Method findSetter(Class classObj, String attribute, Class type)  {	

   	Method mth= findMethod(classObj, "set" + attribute, 1);
	if (mth != null) {
	    Class[] pars = mth.getParameterTypes();
	    if (pars[0].isAssignableFrom(type)) {
		return mth;
	    }
	}
	return null;
    }
    
    /**
     * Finds the setter of a specific attribute without knowing its type.   
     * Returns the method for accessing the attribute, null otherwise
     */    
    Method findSetter(Class classObj, String attribute)  {
	return findMethod(classObj, "set" + attribute, 1) ;
    }
    
   /**
    * Finds a specific constructor of a class
    * Returns the requested constructor or null if not found
    */
    Constructor findConstructor(Class theClass,Class parameterTypes[]) {
	// Get the list of methods		
	Constructor mth = null;
     
	try {
	    mth = theClass.getConstructor(parameterTypes);
	} catch(Exception e) {
	    return null;
	}
	return mth;
    }

   /**
    * Finds a specific method of an object given the number of parameters.
    * Returns the method or null if not found
    */
    Method findMethod(Class classObj, String name, int paramCount) {
 
	Method method = null;
	try {
            Method[] methods=classObj.getMethods();
            int i = 0;
            boolean found = false;
            while ((i < methods.length) && !found) {
                found = methods[i].getName().equals(name);
                if (found) { // Now check if the number of parameters
                    found = methods[i].getParameterTypes().length == paramCount;
                }
                i++;
            }
            if (found) { 
                method = methods[i-1] ; // Note i-1 !
            }
        } catch(Exception e) {
        }
        return method;
   }
    
    /**
     * Get the class of the constructed type name
     * corresponding to the given primitive type
     */

    Class findPrimForClass(Object value) {
	if (value instanceof Boolean)
	    return Boolean.TYPE;
	else if (value instanceof Character)
	    return Character.TYPE;
	else if (value instanceof Byte)
	    return Byte.TYPE;
	else if (value instanceof Short)
	    return Short.TYPE;
	else if (value instanceof Integer)
	    return Integer.TYPE;
     else if (value instanceof Long)
	 return Long.TYPE;
	else if (value instanceof Float)
	    return Float.TYPE;
	else if (value instanceof Double)
	    return Double.TYPE;
	return null;
    }

    /** Get the name of the constructed type name
     * corresponding to the given primitive type
     */    
    Class findClassForPrim(String primName) {
	return (Class) primitiveClasses.get(primName);
    }
      
    /**
     * Converts the array of classes to an array of class signatures.
     */
    static String[] findSignatures(Class[] clz) {
        String signers[] = new String[clz.length];
        for (int i = 0; i < clz.length; i++) {
            signers[i] = findSignature(clz[i]);
        }
        return signers;
    }
    
    /**
     * Converts the class to a class signature.
     */
    static String findSignature(Class clz) {
        return clz.getName();
    }
    
    
    /**
     * Checks if the MBean implements the MBean interface
     */
    
    private Class getMBeanToIntrospect(Class beanClass) throws IntrospectionException {	
	//Check if the MBean implements the MBeanInfo interface
	Class[] interfaces = null;
	Class current = beanClass;       
	boolean isMBean = false;
	Class result = null;
	
	while (current != null) {
	    interfaces = current.getInterfaces();       
	    for (int i=0;i<interfaces.length;i++)  {	  
		
		if (interfaces[i].getName().compareTo(current.getName()+"MBean") == 0) {
		    isMBean = true;
		    break;	 }
	    }
	    if (isMBean) {
		try {
		    result = Class.forName(current.getName()+"MBean");
		    break;
		}
		catch (ClassNotFoundException e) {
		    throw new IntrospectionException("Interface " + beanClass.getName()+"MBean not found");
		}
	    }
	    current = current.getSuperclass();
	}
	return result;
    }

    /**
     * Gets the Class object associated with the class with the given
     * string name.
     */
    private static Class getClass(String s) {
        try {
            return Class.forName(s);
        } catch(ClassNotFoundException e) {   
	    throw new RuntimeErrorException(new Error("ClassNotFoundException for class named" + s));
        }
    }
           
 }
