/*
 * @(#)file      TraceFilter.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.13
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package com.sun.management.jmx;


import javax.management.*;

/**
 * This class is used to filter trace information. The class Trace needs an object of
 * this class for each listener.
 */
public class TraceFilter implements NotificationFilter {

    // -----------------
    // protected variables
    // -----------------
    protected int levels;
    protected int types;

// ------------
// constructors
// ------------
    /**
     * Construct a TraceFilter object with the levels and types selected.
     * Here is an example to specify create a filter:
     * TraceFilter myFilter = new TraceFilter(LEVEL_TRACE, INFO_MBEANSERVER | INFO_TIMER)
     * This filter will select information from MBeanServer and Timer with the level LEVEL_TRACE.
     *
     * @param levels the selected levels of trace information.
     * @param types the selected types of the trace information.
     * @exception IllegalArgumentException Thrown if the specified types or
     * levels are illegal.
     */
    public TraceFilter(int levels, int types) throws IllegalArgumentException {
	if (!Trace.selectedCheck(levels, types))
	    throw new IllegalArgumentException("Illegal levels or types are specified.");
	
	this.levels = levels;
	this.types = types;
    }


// --------------
// public methods
// --------------
    /**
     * Implement the interface NotificationFilter. This implementation uses
     * the specified types and levels to do filtering.
     *
     * @param notification the notification to be sent.
     */
    public boolean isNotificationEnabled(Notification notification) {
	
	boolean ret = false;
	
	if (notification instanceof TraceNotification) {
	    TraceNotification evt = (TraceNotification)notification;
	    
	    if (evt.level == 1 && levels != 0) {
		ret = true;
	    } else {
		ret = ((evt.level & levels) != 0);
	    }
	    
	    ret &= ((evt.type & types) !=0);
	}
	
	return ret;
    }

    /**
     * get the levels selected
     *
     * @return the levels selected for filtering.
     */
    public int getLevels() {
	return levels;
    }
    
    /**
     * get types selected
     *
     * @return the types selected for filtering.
     */
    public int getTypes() {
	return types;
    }
    
}
