/*
 * @(#)file      ClassAttributeValueExp.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   4.6
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management;


/**
 * This class represents the name of the Java implementation class of
 * the MBean. It is used for performing queries based on the class of
 * the MBean.
 */
class ClassAttributeValueExp extends AttributeValueExp   { 


    private String attr;


    /**
     * Basic Constructor.
     */    
    public ClassAttributeValueExp() { 
	attr = "Class";
    } 


    /**
     * Applies the ClassAttributeValueExp on an MBean. Returns the name of
     * the Java implementation class of the MBean.
     *
     * @param name The name of the MBean on which the ClassAttributeValueExp will be applied.
     *
     * @return  The ValueExp.
     *
     * @exception BadAttributeValueExpException 
     * @exception InvalidApplicationException
     */
    public ValueExp apply(ObjectName name) throws BadStringOperationException, BadBinaryOpValueExpException,
	BadAttributeValueExpException, InvalidApplicationException  {

	getAttribute(name);
	Object result = getValue(name);   
	if  (result instanceof String) {
	    return new StringValueExp((String)result);
	} else {
	    throw new BadAttributeValueExpException(result);
	}
    } 
    
    /**
     * Returns the string "Class" representing its value
     */
    public String toString()  { 
	return attr;
    } 
    

    protected Object getValue(ObjectName name) {
	Object result = null;
	try {       
	    // Get the class of the object          
	    result= server.getObjectInstance(name).getClassName();
	} catch (Exception re) {
	}
        return result;
    }   

 }
