/*
 * @(#)file      DescriptorAccess.java
 * @(#)author    IBM Corp.
 * @(#)version   1.6
 * @(#)date      00/04/11
 */
/*
 * (C) Copyright IBM Corp. 1999-2000  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */



package javax.management;


/**
 * This interface is used to gain access to descriptors of the Descriptor class 
 * which are associated with a JMX component, i.e. MBean, MBeanInfo, 
 * MBeanAttributeInfo, MBeanNotificationInfo,
 * MBeanOperationInfo, MBeanParameterInfo.
 * <P>
 * ModelMBeans make extensive use of this interface in ModelMBeanInfo classes.
 */


public interface DescriptorAccess
{
    /** 
    * Returns a copy of Descriptor.
    *
    * @return Descriptor associated with the component implementing this interface.
    * Null should never be returned. At a minimum a default descriptor with the 
    * descriptor name and descriptorType should be returned.
    *
    */
    public Descriptor getDescriptor();
    
    /**
    * Sets Descriptor (full replace).
    *
    * @param Descriptor inDescriptor - replaces the Descriptor associated with the 
    * component implementing this interface. If the inDescriptor is invalid for the
    * type of Info object it is being set for, an exception is thrown.  If the
    * inDescriptor is null, then the Descriptor will revert to its default value
    * which should contain, at a minimum, the descriptor name and descriptorType.
    * 
    */
    public void setDescriptor(Descriptor inDescriptor);
}
