/*
 * @(#)file      MBeanFeatureInfo.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management;


/**
 * Provides general information for an MBean descriptor object. 
 * The feature described can be an attribute, an operation, a parameter, a returned value or a notification.
 */
public class MBeanFeatureInfo implements java.io.Serializable  { 
     

    /**
     * The name of the feature.  
     */
    protected String name = null;
    
    /**
     * The human readable description of the feature.
     */
    protected String description = null;
    
    
    /**
     * Constructs an <CODE>MBeanFeatureInfo</CODE> object.
     *
     * @param name The name of the feature.
     * @param description A human readable description of the feature. 
     */    
    public MBeanFeatureInfo(String name,
			    String description) {
	
    this.name = name;    
    this.description = description;
    
    }
    
    /**
     * Package-private duplicate constructor.
     * <P>
     * This must isolate the new object from any changes to the old object.
     */ 
    MBeanFeatureInfo(MBeanFeatureInfo old) {
        name = old.name;
        description = old.description;
    }
    

    /**
     * Returns the name of the feature.  
     */
    public String getName() {
	return name;
    }
    
    /**
     * Returns the human readable description of the feature.
     */
    public String getDescription() {
	return description;
    }  

 }
