/*
 * @(#)file      MBeanNotificationInfo.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.10
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management;


/**
 * Describes a notification emitted by an MBean.
 */
public class MBeanNotificationInfo extends MBeanFeatureInfo implements Cloneable, java.io.Serializable  { 
  

    /**
     * The type of the notification.  
     */
    private String[] types = null;
    

    /**
     * Constructs an <CODE>MBeanNotificationInfo</CODE> object.
     *
     * @param notifType The event type string (in dot notation).
     * @param name The name of the <CODE>Notification</CODE> class.
     * @param description A human readable description of the data.
     *
     */
    public MBeanNotificationInfo(String[] notifsType,
				 String name,
				 String description) {
	super(name, description);
	this.types = notifsType;
    }
    
    /**
     * Package-private duplicate constructor.
     * <P>
     * This must isolate the new object from any changes to the old object.
     */    
    MBeanNotificationInfo(MBeanNotificationInfo old) {
	super(old.name, old.description);
	types = old.types;
    }

 /**
  * Creates and returns a copy of this object.
  */
  public Object clone () {
        return  new MBeanNotificationInfo(this) ;
  }


    
    /**
     * Returns the notification type string (in dot notation) of the notification.
     */
    public String[] getNotifTypes() {
	if (types != null) {
	    int len = types.length;
	    String[] res = new String[len];
	    for (int i = 0; i < len; i++) {	
		res[i] =  new String(types[i]);
	    }
	    return res;
	} else {
	    return new String[0];
	}
    }
     
 }
