/*
 * @(#)file      MBeanServerDelegate.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.33
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management; 

//Jdk import
import java.util.Random;
import java.util.Properties;

//RI import
import com.sun.management.jmx.JMProperties;
import com.sun.management.jmx.ServiceName;

/**
 * Represents  the Mbean server from the management point of view.
 * The MBeanServerDelegate MBean emits the MBeanServerNotifications when an MBean is 
 * registered/deregistered in the MBean server. 
 */
public class MBeanServerDelegate implements MBeanServerDelegateMBean, NotificationBroadcaster   { 

    /** The MBean server agent identification.*/
    private String mbeanServerId ;
    
    /** The NotificationBroadcasterSupport object that sends the notifications */
    private NotificationBroadcasterSupport broadcaster = null ;

    private static Long oldStamp=new Long(0);

    /**
     * Create a MBeanServerDelegate object.
     */
    public MBeanServerDelegate () {
        String localHost = "";
	Long stamp = null;;
	Properties props = System.getProperties();
	synchronized(oldStamp) {	 
	    stamp = new Long(new java.util.Date().getTime());
	    if (oldStamp.longValue() >= stamp.longValue()) {
		stamp = new Long(oldStamp.longValue() + 1);
	    }
	    oldStamp = stamp;
	}
	try {  
	    localHost= java.net.InetAddress.getLocalHost().getHostName();
	}   
	catch (java.net.UnknownHostException e) {
	}	


        mbeanServerId = new String (localHost + "_" + stamp.toString()) ;

	props.put(JMProperties.JMX_SPEC_NAME, ServiceName.JMX_SPEC_NAME);
	props.put(JMProperties.JMX_SPEC_NAME, ServiceName.JMX_SPEC_NAME);
	props.put(JMProperties.JMX_SPEC_VERSION, ServiceName.JMX_SPEC_VERSION);
	props.put(JMProperties.JMX_SPEC_VENDOR, ServiceName.JMX_SPEC_VENDOR);
	props.put(JMProperties.JMX_IMPL_NAME, ServiceName.JMX_IMPL_NAME);
	props.put(JMProperties.JMX_IMPL_VENDOR, ServiceName.JMX_IMPL_VENDOR);
	props.put(JMProperties.JMX_IMPL_VERSION, ServiceName.JMX_IMPL_VERSION);
	
	System.setProperties(props);
        broadcaster = new NotificationBroadcasterSupport() ;
    }
    

    /**
     * Returns the MBean server agent identification
     */
    public String getMBeanServerId() {
	return mbeanServerId;
    }
    
    /**
     * Returns the full name of the JMX specification implemented
     * by this product.      
     */    
    public String getSpecificationName() {
	return System.getProperty(JMProperties.JMX_SPEC_NAME);
    }

    /**
     * Returns the version of the JMX specification implemented
     * by this product.      
     */    
    public String getSpecificationVersion() {
	return System.getProperty(JMProperties.JMX_SPEC_VERSION);
    }

    /**
     * Returns the vendor of the JMX specification implemented
     * by this product.      
     */    
    public String getSpecificationVendor() {
	return System.getProperty(JMProperties.JMX_IMPL_VENDOR);
    }

    /**
     * Returns the JMX implementation name (the name of this product).     
     */    
    public String getImplementationName() {
	return System.getProperty(JMProperties.JMX_IMPL_NAME);
    }

    /**
     * Returns the JMX implementation version (the version of this product).     
     */    
    public String getImplementationVersion() {
	return System.getProperty(JMProperties.JMX_IMPL_VERSION);
    }

    /**
     * Returns the JMX implementation vendor (the vendor of this product).     
     */    
    public String getImplementationVendor()  {
	return System.getProperty(JMProperties.JMX_IMPL_VENDOR);
    }
    
    
    /**
     * Enables to add a listener to a registered MBean.
     *
     * @param listener The listener object which will handle the notifications emitted by the registered MBean.
     * @param filter The filter object. If filter is null, no filtering will be performed before handling notifications.
     * @param handback The context to be sent to the listener when a notification is emitted.
     *
     * @exception IllegalArgumentException Listener parameter is null.
     */
    public synchronized void addNotificationListener(NotificationListener listener, NotificationFilter filter, Object handback) 
        throws IllegalArgumentException {
        broadcaster.addNotificationListener(listener,filter,handback) ;
    }
    
    /**
     * Enables to remove a listener from a registered MBean.
     *
     * @param name The name of the MBean on which the listener should be removed.
     * @param listener The listener object which will handle the notifications emitted by the registered MBean.
     * This method will remove all the information related to this listener.
     *
     * @exception ListenerNotFoundException The listener is not registered in the MBean.
     */
    public synchronized void removeNotificationListener(NotificationListener listener)
        throws ListenerNotFoundException {
        broadcaster.removeNotificationListener(listener) ;
    }

    /**    
     * Returns a NotificationInfo object contaning the name of the Java class of the notification
     * and the notification types sent.
     */
    public MBeanNotificationInfo[] getNotificationInfo() {
        String[] types  = { MBeanServerNotification.UNREGISTRATION_NOTIFICATION,
                            MBeanServerNotification.REGISTRATION_NOTIFICATION};
        MBeanNotificationInfo[] notifsInfo = { new MBeanNotificationInfo(types, "MBeanServerNotification", 
                                                                         "Notifications sent by the MBeanServerDelegate MBean")};
        return notifsInfo;
    }  

    /**
     * Enables the MBean server to send a notification.
     *   
     *  @param notification The notification to send.
     */
    public synchronized void sendNotification(Notification notification) {      
        broadcaster.sendNotification(notification);
    }

}
