/*
 * @(#)file      MBeanServerFactory.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.12
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management;

// java import
import java.util.Iterator ;
    
// RI import
import com.sun.management.jmx.MBeanServerImpl ;
import com.sun.management.jmx.ServiceName ;

/**
* Provides MBean server references.
*/

public class MBeanServerFactory { 

    // private constructor defined to "hide" the default public constructor
    private MBeanServerFactory() {
	
    }

    /**
     * Remove internal references to a created MBeanServer. This allows the java garbage-collector
     * to remove Mbean server object, when necessary.
     *
     * @param mbeanServer The MbeanServer object to remove.
     *
     * @exception java.lang.IllegalArgumentException The mbeanServer is not found.
     */
     public static void releaseMBeanServer (MBeanServer mbeanServer) {
	removeMBeanServer(mbeanServer) ;
     }

    /**
     * Return a new object implementing the MBeanServer interface with a standard default domain name.
     * The default domain name is used as the domain part in the ObjectName of MBeans.
     * <p>
     * The standard default domain name is defined in {@link com.sun.management.jmx.ServiceName#DOMAIN ServiceName.DOMAIN}
     * <p>
     * The MBeanServer reference is internally kept. This will allows <CODE>findMBeanServer</CODE> to
     * give back a reference to this newly MBeanServer object.
     */
     public static MBeanServer createMBeanServer () {
	MBeanServer mbeanServer = new MBeanServerImpl() ;
	addMBeanServer(mbeanServer) ;
	return mbeanServer ;
     }

    /**
     * Return a new object implementing the MBeanServer interface with the specified default domain name.
     * The default domain name is used as the domain part in the ObjectName
     * of MBeans if domain is specified by the user is null.
     * <p>
     * The MBeanServer reference is internally kept. This will allows <CODE>findMBeanServer</CODE> to
     * give back a reference to this newly MBeanServer object.
     */  
    public static MBeanServer createMBeanServer (String domain)  {
	MBeanServer mbeanServer = new MBeanServerImpl(domain) ;
	addMBeanServer(mbeanServer) ;
	return mbeanServer ;
     }

    /**
     * Return a new object implementing the MBeanServer interface with a standard default domain name, without keeping
     * an internal reference to this new object.
     * The default domain name is used as the domain part in the ObjectName of MBeans.
     * <p>
     * The standard default domain name is defined in {@link com.sun.management.jmx.ServiceName#DOMAIN ServiceName.DOMAIN}
     * <p>
     * No reference is kept. <CODE>findMBeanServer</CODE> will not be able to
     * give back reference to this newly MBeanServer object, but the java garbage-collector
     * will be able to remove Mbean server object, when necessary.
     */
     public static MBeanServer newMBeanServer () {
	MBeanServer mbeanServer = new MBeanServerImpl() ;
	return mbeanServer ;
     }

    /**
     * Return a new object implementing the MBeanServer interface with the specified default domain name, without keeping
     * an internal reference to this new object.
     * The default domain name is used as the domain part in the ObjectName
     * of MBeans if domain is specified by the user is null.
     * <p>
     * No reference is kept. <CODE>findMBeanServer</CODE> will not be able to
     * give back reference to this newly MBeanServer object, but the java garbage-collector
     * will be able to remove Mbean server object, when necessary.
     */  
    public static MBeanServer newMBeanServer (String domain)  {
	MBeanServer mbeanServer = new MBeanServerImpl(domain) ;
	return mbeanServer ;
     }
 
    /**  
     * Return a list of objects implementing the MBeanServer interface.
     * This static method allows a user to retrieve references on MBeanServer which have been instantiated
     * in the Java Virtual Machine.
     *   
     * @param AgentId The agent identifier of the MBeanServer to retrieve. If this parameter is null,
     * all MBeanServers present in the JVM are returned.
     *   
     * @return  A list of MBeanServer objects.     
     *    
     */   
 
    public synchronized static java.util.ArrayList findMBeanServer (String AgentId) {
        if ( AgentId == null ) {
            return (java.util.ArrayList)MBeanServerList.clone() ;
        }
        java.util.ArrayList result = new java.util.ArrayList() ;
        try {
            ObjectName delegateName = new ObjectName(ServiceName.DELEGATE) ;
            String agentName = null ;
            for (Iterator i  = MBeanServerList.iterator(); i.hasNext(); ) {
                MBeanServer MBeanServerElm = (MBeanServer) i.next();
                try {
                    agentName = (String) MBeanServerElm.getAttribute(delegateName,"MBeanServerId") ;
                    if (agentName.equals(AgentId)) {
                        result.add(MBeanServerElm) ;
                    }
                }
		catch (Exception e ) {
                }
            }    
        }                                           
        catch (Exception e ) {
        }
        return result ;
    }

    private synchronized static void addMBeanServer(MBeanServer mbeanserver) {
        MBeanServerList.add(mbeanserver) ;
    }

    private synchronized static void removeMBeanServer(MBeanServer mbeanserver) {
	try
	{
        	MBeanServerList.remove(MBeanServerList.indexOf(mbeanserver)) ;
	}
	catch ( IndexOutOfBoundsException e)
	{
		throw new java.lang.IllegalArgumentException() ;
	}

    }

    private static java.util.ArrayList MBeanServerList = new java.util.ArrayList() ;


}
