/*
 * @(#)file      NumericValueExp.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   4.5
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management;


/**
 * This class represents numbers that are arguments to relational constraints.
 * A NumericValueExp may be used anywhere a ValueExp is required.
 */
class NumericValueExp extends ValueExpSupport   { 


    private long longVal;

    private double doubleVal;

    private boolean valIsLong;

        
    /**
     * Basic constructor.
     */   
    public NumericValueExp() { 
    } 

    /** Creates a new NumericValue representing the numeric literal <val>.*/    
    NumericValueExp(Number val) {
	if (val instanceof Long ||
	    val instanceof Integer) {
	    this.longVal = val.longValue();
	    valIsLong	 = true;
	    
	} else {
	    this.doubleVal = val.doubleValue();
	}
    }
    

    /**
     * Returns a double numeric value
     */
    public double doubleValue()  { 
	if (valIsLong) {
	    return (double)longVal;
	} else {
	    return doubleVal;
	}
    } 

    /**
     * Returns a long numeric value
     */
    public long longValue()  { 
	if (valIsLong) {
	    return longVal;
	} else {
	    return (long)doubleVal;
	}
    } 

    /**
     * Returns true is if the numeric value is a long, false otherwise.
     */
    public boolean isLong()  { 
	return valIsLong;
    } 
    
    /**
     * Returns the string representing the object
     */    
    public String toString()  { 
	if (valIsLong) {
	    return String.valueOf(longVal);
	} else {
	    return String.valueOf(doubleVal);
	}
    } 
 }
