/*
 * @(#)file      StringValueExp.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   4.7
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management;



/**
* Represents strings that are arguments to relational constraints.
* A <CODE>StringValueExp</CODE> may be used anywhere a <CODE>ValueExp</CODE> is required.
*/
public class StringValueExp implements ValueExp   { 
    

    private String val;

    static MBeanServer server;

    /**
     * Basic constructor.
     */    
    public StringValueExp() { 
    } 

    /**
     * Creates a new <CODE>StringValueExp</CODE> representing the string literal val.
     *
     */
    public StringValueExp(String val) { 
	this.val = val;
    } 

    /**
     * Returns the string represented by the <CODE>StringValueExp</CODE> instance.
     */   
    public String getValue()  { 
	return val;
    } 

    /**
     * Returns the string representing the object.
     */
    public String toString()  { 
	return "'" + val + "'";
    } 
   

    /**
     * Sets the MBean server on which the query is to be performed.
     *
     * @param s The MBean server on which the query is to be performed.
     */
    public void setMBeanServer(MBeanServer s)  { 
	server = s;
    }        

    /**
     * Applies the ValueExp on a MBean.
     *
     * @param name The name of the MBean on which the ValueExp will be applied.
     *
     * @return  The <CODE>ValueExp</CODE>.
     *
     * @exception BadStringOperationException
     * @exception BadBinaryOpValueExpException
     * @exception BadAttributeValueExpException 
     * @exception InvalidApplicationException
     */   
    public ValueExp apply(ObjectName name) throws BadStringOperationException, BadBinaryOpValueExpException,
	BadAttributeValueExpException, InvalidApplicationException  { 
	return this;
    } 
 }
