/*
 * @(#)file      InvalidTargetObjectTypeException.java
 * @(#)author    IBM Corp.
 * @(#)version   1.7
 * @(#)date      00/04/11
 */
/*
 * Copyright IBM Corp. 1999-2000.  All rights reserved.
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. IBM will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */


package javax.management.modelmbean;

/**
 * Exception thrown when an invalid target object type is specified.
 *    
 */

public class InvalidTargetObjectTypeException  extends Exception
{

    String msgStr;
    Exception relatedExcept;


    /**
     * Default constructor 
     */
    public InvalidTargetObjectTypeException ()
    {
        relatedExcept = null;
        msgStr =  "InvalidTargetObjectTypeException: ";
    }


    /**
     * Constructor from a string.
     *
     * @param s String value that will be incorporated in the message for
     *    this exception. 
     */

    public InvalidTargetObjectTypeException (String s)
    {
        msgStr = new String("InvalidTargetObjectTypeException: " + s);
    }


    /**
     * Constructor taking an exception and a string.
     *
     * @param e Exception that we may have caught to reissue as an
     *    InvalidTargetObjectTypeException.  The message will be used, and we may want to
     *    consider overriding the printStackTrace() methods to get data
     *    pointing back to original throw stack. 
     * @param s String value that will be incorporated in message for
     *    this exception. 
     */

    public InvalidTargetObjectTypeException (Exception e, String s)
    {
        relatedExcept = e;
        if (e != null) {
            msgStr = new String("InvalidTargetObjectTypeException: " + s 
                                + "\n\t triggered by:" + e.toString() );
        }
        else {
            msgStr = new String("InvalidTargetObjectTypeException: " + s );
        }
    }

    /**
     * Override default toString
     */
    public String toString() {
        return new String(msgStr);
    }

    /**
     * Override <i>Throwables</i> getMessage
     */
    public String getMessage() {
        return new String(msgStr);
    }
}

