/*
 * @(#)file      ModelMBean.java
 * @(#)author    IBM Corp.
 * @(#)version   1.8
 * @(#)date      00/04/11
 */
/*
 * Copyright IBM Corp. 1999-2000.  All rights reserved.
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. IBM will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */



package javax.management.modelmbean;

import javax.management.*;

import javax.management.InstanceNotFoundException;

/**                    
 * This interface must be implemented by the ModelMBeans. An implementation of this interface 
 * must be shipped with every JMX Agent. 
 * <P>
 * Java resources wishing to be manageable instatiate the ModelMBean using the MBeanServer's 
 * createMBean method.  The resource then sets the ModelMBeanInfo (with Descriptors) for the ModelMBean
 * instance. The attributes and operations exposed via the ModelMBeanInfo for the ModelMBean are accessible
 * from Mbeans, connectors/adapters like other MBeans. Through the ModelMBeanInfo Descriptors, values and methods in
 * the managed application can be defined and mapped to attributes and operations of the ModelMBean.
 * This mapping can be defined during development in an XML formatted file or dynamically and
 * programmatically at runtime. 
 * <P>
 * Every ModelMBean which is instantiated in the MBeanServer becomes manageable: 
 * its attributes and operations
 * become remotely accessible through the connectors/adaptors connected to that MBeanServer. 
 * A Java object cannot be registered in the MBeanServer unless it is a JMX compliant MBean.
 * By instantiating a ModelMBean, resources are guaranteed that the MBean is valid.
 * <P>
 * MBeanException and RuntimeOperatiosException must be thrown on every public method.  This allows 
 * for wrappering exceptions from distributed communications (RMI, EJB, etc.).  These exceptions do
 * not have to be thrown by teh implementation except in the scenarios described in the specification 
 * and javadoc.
 */

public interface ModelMBean extends 
	 DynamicMBean,
	 PersistentMBean,
	 ModelMBeanNotificationBroadcaster 
{

	/**
	 * Initializes a ModelMBean object using ModelMBeanInfo passed in. 
	 * The ModelMBean must be instantiated, but not registered with the MBeanServer.
	 * After the ModelMBean's ModelMBeanInfo (with Descriptors) are customized, the ModelMBean
	 * should be registered with the MBeanServer.
	 * <P>
	 *
	 * @param inModelMBeanInfo The ModelMBeanInfo object to be used by the ModelMBean.    
	 *
	 *
	 * @exception MBeanException The constructor of the MBeanInfo has return null
	 *           or thrown an exception.
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException: 
	 *       The MBeanInfo passed in parameter is null or invalid.
	 *
	 */

	public void setModelMBeanInfo(ModelMBeanInfo inModelMBeanInfo) 
	throws MBeanException, RuntimeOperationsException;

	/**
	 * Sets the instance handle of the object against which to execute all methods in this ModelMBean
	 * management interface (MBeanInfo and Descriptors) 
	 *
	 * @param mr Object that is the managed resource
	 * @param mr_type The type of reference for the managed resource.  Can be: ObjectReference,
	 *               Handle, IOR, EJBHandle, RMIReference.
	 *               If the MBeanServer cannot process the mr_type passed in, an InvalidTargetTypeException
	 *               will be thrown.
	 *     
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException: 
	 *       The managed resource or managed resoure type passed in parameter is null or invalid.
	 * @exception InstanceNotFoundException The managed resource object could not be found
	 * @exception InvalidTargetObjectType The managed resource type cannot be processed by the 
	 * ModelMBean or JMX Agent.
	 */

	public void setManagedResource(Object mr, String mr_type)
	throws MBeanException, RuntimeOperationsException, 
		 InstanceNotFoundException, InvalidTargetObjectTypeException ;

}
