/*
 * @(#)file      RoleResult.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management.relation;

import java.util.Iterator;

import java.io.Serializable;

/**
 * Represents the result of a multiple access to several roles of a relation
 * (either for reading or writing).
 */
public class RoleResult implements Serializable {

    //
    // Private members
    //

    // List of roles successfully accessed
    private RoleList myRoleList = null;

    // List of roles unsuccessfully accessed
    private RoleUnresolvedList myRoleUnresList = null;

    //
    // Constructor
    //

    /**
     * Constructor
     *
     * @param theRoleList  list of roles succesfully accessed
     * @param theRoleUnresList  list of roles not accessed (with problem
     * descriptions)
     */
    public RoleResult(RoleList theRoleList,
		      RoleUnresolvedList theRoleUnresList) {

	setRoles(theRoleList);
	setRolesUnresolved(theRoleUnresList);
	return;
    }

    //
    // Accessors
    //

    /**
     * Retrieves list of roles successfully accessed
     *
     * @return a RoleList
     */
    public RoleList getRoles() {
	return myRoleList;
    }

    /**
     * Retrieves list of roles unsuccessfully accessed
     *
     * @return a RoleUnresolvedList
     */
    public RoleUnresolvedList getRolesUnresolved() {
	return myRoleUnresList;
    }

    /**
     * Sets list of roles successfully accessed
     *
     * @param theRoleList  list of roles successfully accessed
     */
    public void setRoles(RoleList theRoleList) {
	if (theRoleList != null) {

	    myRoleList = new RoleList();

	    for (Iterator roleIter = theRoleList.iterator();
		 roleIter.hasNext();) {
		Role currRole = (Role)(roleIter.next());
		myRoleList.add((Role)(currRole.clone()));
	    }
	} else {
	    myRoleList = null;
	}
	return;
    }

    /**
     * Sets list of roles unsuccessfully accessed
     *
     * @param theRoleUnresolvedList  list of roles unsuccessfully accessed
     */
    public void setRolesUnresolved(RoleUnresolvedList theRoleUnresList) {
	if (theRoleUnresList != null) {

	    myRoleUnresList = new RoleUnresolvedList();

	    for (Iterator roleUnresIter = theRoleUnresList.iterator();
		 roleUnresIter.hasNext();) {
		RoleUnresolved currRoleUnres =
		    (RoleUnresolved)(roleUnresIter.next());
		myRoleUnresList.add((RoleUnresolved)(currRoleUnres.clone()));
	    }
	} else {
	    myRoleUnresList = null;
	}
	return;
    }
}
