/*
 * @(#)file      RoleUnresolvedList.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management.relation;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

/**
 * A RoleUnresolvedList represents a list of RoleUnresolved objects,
 * representing roles not retrieved from a relation due to a problem
 * encountered whe trying to access (read or write to roles)
 */
public class RoleUnresolvedList extends ArrayList {

    //
    // Constructors
    //

    /**
     * Constructs an empty RoleUnresolvedList.
     */
    public RoleUnresolvedList() {
	super();
	return;
    }

    /**
     * Constructs an empty RoleUnresolvedList with the initial capacity
     * specified.
     *
     * @param theInitialCapacity  initial capacity
     */
    public RoleUnresolvedList(int theInitialCapacity) {
	super(theInitialCapacity);
	return;
    }

    /**
     * Constructs a RoleUnresolvedList containing the elements of the
     * ArrayList specified, in the order in which they are returned
     * by the ArrayList's iterator. The RoleUnresolvedList instance has
     * an initial capacity of 110% of the size of the ArrayList
     * specified.
     *
     * @param theList  list of RoleUnresolved objects
     *
     * @exception IllegalArgumentException  if:
     * <P>- null parameter
     * <P>or
     * <P>- an element in the ArrayList is not a RoleUnresolved
     */
    public RoleUnresolvedList(List theList)
	throws IllegalArgumentException {

	if (theList == null) {
	    // Revisit [cebro] Localize message
	    String excMsg = "Invalid parameter.";
	    throw new IllegalArgumentException(excMsg);
	}

	int i = 0;
	for (Iterator eltIter = theList.iterator();
	     eltIter.hasNext();) {
	    Object currElt = eltIter.next();
	    if (!(currElt instanceof RoleUnresolved)) {
		StringBuffer excMsgStrB = new StringBuffer();
		// Revisit [cebro] Localize message
		String excMsg = "An element is not a RoleUnresolved at index ";
		excMsgStrB.append(excMsg);
		excMsgStrB.append(i);
		throw new IllegalArgumentException(excMsgStrB.toString());
	    }
	    i++;
	    super.add(currElt);
	}
	return;
    }

    //
    // Accessors
    //

    /**
     * Adds the RoleUnresolved specified as the last element of the list.
     *
     * @param theRoleUnres - the unresolved role to be added.
     *
     * @exception IllegalArgumentException  if the unresolved role is null.
     */
    public void add(RoleUnresolved theRoleUnres)
	throws IllegalArgumentException {

	if (theRoleUnres == null) {
	    // Revisit [cebro] Localize message
	    String excMsg = "Invalid parameter.";
	    throw new IllegalArgumentException(excMsg);
	}
	super.add(theRoleUnres);
	return;
    }

    /**
     * Inserts the unresolved role specified as an element at the position
     * specified.
     * Elements with an index greater than or equal to the current position are
     * shifted up.
     *
     * @param theIndex - The position in the list where the new
     * RoleUnresolved object is to be inserted.
     * @param theRoleUnresolved - The RoleUnresolved object to be inserted.
     *
     * @exception IllegalArgumentException  if the unresolved role is null.
     */
    public void add(int theIndex,
		    RoleUnresolved theRoleUnres)
	throws IllegalArgumentException,
	       IndexOutOfBoundsException {

	if (theRoleUnres == null) {
	    // Revisit [cebro] Localize message
	    String excMsg = "Invalid parameter.";
	    throw new IllegalArgumentException(excMsg);
	}

	super.add(theIndex, theRoleUnres);
	return;
    }

    /**
     * Sets the element at the position specified to be the unresolved role
     * specified.
     * The previous element at that position is discarded.
     *
     * @param theIndex - The position specified.
     * @param theRoleUnres - The value to which the unresolved role element
     * should be set.
     *
     * @exception IllegalArgumentException   if the unresolved role is null.
     */
     public void set(int theIndex,
		     RoleUnresolved theRoleUnres)
	 throws IllegalArgumentException,
                IndexOutOfBoundsException {

	if (theRoleUnres == null) {
	    // Revisit [cebro] Localize message
	    String excMsg = "Invalid parameter.";
	    throw new IllegalArgumentException(excMsg);
	}

	super.set(theIndex, theRoleUnres);
	return;
     }

    /**
     * Appends all the elements in the RoleUnresolvedList specified to the end
     * of the list, in the order in which they are returned by the Iterator of
     * the RoleUnresolvedList specified.
     *
     * @param theRoleUnresolvedList - Elements to be inserted into the list
     * (can be null).
     *
     * @exception IndexOutOfBoundsException  if accessing with an index
     * outside of the list.
     */
    public boolean addAll(RoleUnresolvedList theRoleUnresolvedList)
	throws IndexOutOfBoundsException {

	if (theRoleUnresolvedList == null) {
	    return true;
	}

	return (super.addAll(theRoleUnresolvedList));
    }

    /**
     * Inserts all of the elements in the RoleUnresolvedList specified into
     * this list, starting at the specified position, in the order in which
     * they are returned by the Iterator of the RoleUnresolvedList specified.
     *
     * @param theIndex - Position at which to insert the first element from the
     * RoleUnresolvedList specified.
     * @param theRoleUnresolvedList - Elements to be inserted into the list.
     *
     * @exception IllegalArgumentException  if the role is null.
     * @exception IndexOutOfBoundsException  if accessing with an index
     * outside of the list.
     */
    public boolean addAll(int theIndex,
			  RoleUnresolvedList theRoleUnresolvedList)
	throws IllegalArgumentException,
               IndexOutOfBoundsException {

	if (theRoleUnresolvedList == null) {
	    // Revisit [cebro] Localize message
	    String excMsg = "Invalid parameter.";
	    throw new IllegalArgumentException(excMsg);
	}

	return (super.addAll(theIndex, theRoleUnresolvedList));
    }

    /**
     * Cloning
     * <P>Returns a new RoleList, but no copy of the included elements.
     */
    public Object clone() {
	ArrayList array = (ArrayList)(super.clone());
	return new RoleList(array);
    }
}
