/*
 * @(#)file      TimerNotification.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)date      00/04/11
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 * Ce logiciel est propriet de Sun Microsystems, Inc.
 * Distribu par des licences qui en restreignent l'utilisation. 
 */

package javax.management.timer; 

/**
 * This class provides definitions of the notifications sent by timer MBeans.
 * <BR>It defines a timer notification identifier which allows to retreive a timer notification
 * from the list of notifications of a timer MBean.
 * <P>
 * The timer notifications are created and handled by the timer MBean.
 *
 * @version     1.7     02/04/00
 * @author      Sun Microsystems, Inc
 */

public class TimerNotification extends javax.management.Notification { 


    /*
     * ------------------------------------------
     *  PRIVATE VARIABLES
     * ------------------------------------------
     */
    
    /**
     * Timer notification identifier.
     * This identifier is used to retreive a timer notification from the timer list of notifications.
     */
    private Integer notificationID;

    
    /*
     * ------------------------------------------
     *  CONSTRUCTORS
     * ------------------------------------------
     */
    
    /**
     * Creates a timer notification object.
     *
     * @param type The notification type.
     * @param source The notification producer.
     * @param sequenceNumber The notification sequence number within the source object.
     * @param timeStamp The notification emission date.
     * @param msg The notification message.
     * @param id The notification identifier.
     */
    TimerNotification(String type, Object source, long sequenceNumber, long timeStamp, String msg, Integer id) {
        
        super(type, source, sequenceNumber, timeStamp, msg);
        this.notificationID = id;
    }
    
    /*
     * ------------------------------------------
     *  PUBLIC METHODS
     * ------------------------------------------
     */
    
    // GETTERS AND SETTERS
    //--------------------    
    
    /**
     * Gets the identifier of this timer notification.
     *
     * @return The identifier.
     */
    public Integer getNotificationID() { 
        return notificationID;
    } 

    /*
     * ------------------------------------------
     *  PACKAGE METHODS
     * ------------------------------------------
     */
    
    /**
     * Creates and returns a copy of this object.
     *
     * @param id The notification identifier.
     */
    Object cloneTimerNotification() { 
        
        TimerNotification clone = new TimerNotification(this.getType(), this.getSource(), this.getSequenceNumber(), 
                                                        this.getTimeStamp(), this.getMessage(), notificationID);
        clone.setUserData(this.getUserData());
        return clone;
    } 
}
